@@ -1,7 +1,10 @@
+use noodles_bgzf as bgzf;
 use noodles_vcf as vcf;
-use structopt::StructOpt;
 use vcf::record::chromosome::Chromosome;
 use vcf::record::filters::Filters;
+use vcf::record::info::field::{Key, Value};
+
+use structopt::StructOpt;
 
 use std::error::Error;
 use std::fs::File;
@@ -12,6 +15,7 @@ mod cli;
 use cli::Opt;
 
 const HEADER_START: [&str; 7] = ["contig", "pos", "id", "ref", "alt", "qual", "filter"];
+const VEP_DESC_PREFIX: &str = "Consequence annotations from Ensembl VEP. Format: ";
 
 enum TableKind {
     Cartesian,
@@ -27,7 +31,14 @@ fn main() -> Result<(), Box<dyn Error>> {
     };
 
     let input: Box<dyn Read> = match opt.input.clone() {
+        Some(path)
+	    // gotta be a better way; ==Some("gz") doesn't work
+            if opt.bgzip || (path.extension().is_some() && path.extension().unwrap() == "gz") =>
+        {
+            Box::new(bgzf::Reader::new(File::open(path)?))
+        }
         Some(path) => Box::new(File::open(path)?),
+        None if opt.bgzip => Box::new(bgzf::Reader::new(stdin())),
         None => Box::new(stdin()),
     };
 
@@ -71,27 +82,58 @@ fn convert_records(
                 Some(d) => d,
                 None => "_".to_string(),
             };
-            convert_records_cartesian(reader, writer, header, sample_delim, info_prefix)?
+            convert_records_cartesian(
+                reader,
+                writer,
+                header,
+                sample_delim,
+                info_prefix,
+                opt.vep_fields,
+            )?
+        }
+        TableKind::Stacked => {
+            convert_records_stacked(reader, writer, header, info_prefix, opt.vep_fields)?
         }
-        TableKind::Stacked => convert_records_stacked(reader, writer, header, info_prefix)?,
     }
     Ok(())
 }
 
+// unneccesary code duplication between this + _cartesian
 fn convert_records_stacked(
     mut reader: vcf::Reader<BufReader<impl Read>>,
     mut writer: csv::Writer<impl Write>,
     header: vcf::Header,
     info_prefix: String,
+    split_csq: bool,
 ) -> Result<(), Box<dyn Error>> {
     let fmts = header.formats();
     let infos = header.infos();
     let samples = header.sample_names();
 
-    let mut info_header: Vec<String> = infos
-        .keys()
-        .map(|field| format!("{}{}", info_prefix, field))
-        .collect();
+    let mut info_header = Vec::new();
+    let mut number_csq = 0;
+
+    for key in infos.keys() {
+        if split_csq {
+            match key {
+                Key::Other(name, ..) if name == "CSQ" => {
+                    let (_, fields) = infos
+                        .get(key)
+                        .unwrap()
+                        .description()
+                        .split_at(VEP_DESC_PREFIX.len());
+                    let mut csq_fields: Vec<String> =
+                        fields.split('|').map(ToString::to_string).collect();
+                    number_csq = csq_fields.len() + 1;
+                    info_header.append(&mut csq_fields);
+                    info_header.push("CSQ_other_transcripts".to_owned());
+                }
+                _ => info_header.push(format!("{}{}", info_prefix, key)),
+            }
+        } else {
+            info_header.push(format!("{}{}", info_prefix, key));
+        }
+    }
 
     let mut out_header: Vec<String> = HEADER_START.map(ToString::to_string).to_vec();
     out_header.append(&mut info_header);
@@ -106,12 +148,12 @@ fn convert_records_stacked(
     out_header.append(&mut fmt_header);
     writer.write_record(&out_header)?;
 
-    for result in reader.records() {
+    for result in reader.records(&header) {
         let record = result?;
 
         let mut locus_fields = main_fields_from_record(&record);
 
-        push_info_fields(infos, record.info(), &mut locus_fields);
+        push_info_fields(infos, record.info(), &mut locus_fields, number_csq);
 
         let sample_fmts = record.genotypes();
 
@@ -137,15 +179,36 @@ fn convert_records_cartesian(
     header: vcf::Header,
     sample_delim: String,
     info_prefix: String,
+    split_csq: bool,
 ) -> Result<(), Box<dyn Error>> {
     let fmts = header.formats();
     let infos = header.infos();
     let samples = header.sample_names();
 
-    let mut info_header: Vec<String> = infos
-        .keys()
-        .map(|field| format!("{}{}", info_prefix, field))
-        .collect();
+    let mut info_header = Vec::new();
+    let mut number_csq = 0;
+
+    for key in infos.keys() {
+        if split_csq {
+            match key {
+                Key::Other(name, ..) if name == "CSQ" => {
+                    let (_, fields) = infos
+                        .get(key)
+                        .unwrap()
+                        .description()
+                        .split_at(VEP_DESC_PREFIX.len());
+                    let mut csq_fields: Vec<String> =
+                        fields.split('|').map(ToString::to_string).collect();
+                    number_csq = csq_fields.len() + 1;
+                    info_header.append(&mut csq_fields);
+                    info_header.push("CSQ_other_transcripts".to_owned());
+                }
+                _ => info_header.push(format!("{}{}", info_prefix, key)),
+            }
+        } else {
+            info_header.push(format!("{}{}", info_prefix, key));
+        }
+    }
 
     let mut out_header: Vec<String> = HEADER_START.map(ToString::to_string).to_vec();
     out_header.append(&mut info_header);
@@ -161,13 +224,13 @@ fn convert_records_cartesian(
 
     out_header.append(&mut fmt_header);
     writer.write_record(&out_header)?;
-
-    for result in reader.records() {
+    dbg!(&out_header.len());
+    for result in reader.records(&header) {
         let record = result?;
 
         let mut fields = main_fields_from_record(&record);
 
-        push_info_fields(infos, record.info(), &mut fields);
+        push_info_fields(infos, record.info(), &mut fields, number_csq);
 
         let sample_fmts = record.genotypes();
 
@@ -181,7 +244,7 @@ fn convert_records_cartesian(
             }
         }
 
-        writer.write_record(&fields)?;
+        writer.write_record(&fields).expect("fuck");
     }
     Ok(())
 }
@@ -190,15 +253,38 @@ fn push_info_fields(
     infos: &vcf::header::Infos,
     info: &vcf::record::Info,
     fields: &mut Vec<String>,
+    split_csq: usize,
 ) {
     for info_field in infos.keys() {
-        fields.push(match info.get(info_field) {
-            Some(field) => match field.value() {
-                vcf::record::info::field::Value::Flag => "true".to_string(),
-                _ => field.value().to_string(),
-            },
-            None => "NA".to_owned(),
-        })
+        match info_field {
+            // csq field and we want to split it
+            Key::Other(ref name, ..) if split_csq != 0 && name == "CSQ" => {
+                let field = info
+                    .get(info_field)
+                    .expect("No CSQ field!")
+                    .value()
+                    .expect("No CSQ value!");
+                let mut subfields: Vec<String> = match field {
+                    Value::String(s) => s.splitn(split_csq, '|').map(ToString::to_string).collect(),
+                    _ => panic!(),
+                };
+                if subfields.len() < split_csq {
+                    for _ in 0..(split_csq - subfields.len()) {
+                        subfields.push("NA".to_owned());
+                    }
+                }
+                fields.append(&mut subfields);
+            }
+            // single field
+            _ => fields.push(match info.get(info_field) {
+                None => "NA".to_owned(),
+                Some(field) => match field.value() {
+                    Some(vcf::record::info::field::Value::Flag) => "true".to_string(),
+                    Some(f) => f.to_string(),
+                    None => "missing".to_string(),
+                },
+            }),
+        }
     }
 }
 
